from typing import Literal, Union, Tuple
from PIL import Image
import customtkinter as CTk

def resolve_padding(padding: Union[int, Tuple[int, int]], index: int) -> int:
    if isinstance(padding, int):
        return padding
    elif isinstance(padding, tuple) and len(padding) == 2:  
        return padding[0 if index == 0 else 1]
    else:
        raise ValueError("Padding must be an int or a tuple of two ints")

def colorize_image(master: CTk.CTkBaseClass, image: Image, color, appearance_mode: Literal['light', 'dark'] = 'light') -> Image:
    if isinstance(color, list):
        color = color[0] if appearance_mode == 'light' else color[1]
    # Create a colored version for normal state
    r, g, b = parse_tk_color(master, color)

    # Convert image to RGBA
    img_rgba = image.convert("RGBA")
    
    # Create normal state image
    normal_data = []
    for item in img_rgba.getdata():
        if item[3] > 0:  # If pixel is not fully transparent
            normal_data.append((r, g, b, item[3]))
        else:
            normal_data.append(item)
    
    colorized_image = Image.new("RGBA", img_rgba.size)
    colorized_image.putdata(normal_data)
    return colorized_image

def parse_tk_color(master: CTk.CTkBaseClass, color):
    # Tk Color to RGB tuple
    try:
        rgb = master.winfo_rgb(color)
        return (rgb[0] // 256, rgb[1] // 256, rgb[2] // 256)
    except Exception:
        pass

    # Handle hex colors
    if color.startswith("#") and len(color) == 7:
        r = int(color[1:3], 16)
        g = int(color[3:5], 16)
        b = int(color[5:7], 16)
        return (r, g, b)

    raise ValueError(f"Invalid color format: {color}")


from typing import Optional, Union
import customtkinter as CTk
import tkinter
from PIL import Image
import sys
from .theme import CTkSidebarTheme
from .util import resolve_padding, colorize_image

class CTkSidebarItem(CTk.CTkBaseClass):
    def __init__(self,
                 master,
                 theme : CTkSidebarTheme,
                 id: Optional[Union[int, str]]=None,
                 text : str="",
                 width=200,
                 icon : Optional[Union[Image.Image, tuple[CTk.CTkImage, CTk.CTkImage]]]=None,
                 icon_size=(20,20),
                 has_submenu : bool = False,
                 submenu_expanded : bool = True,
                 override_text_x : Optional[int]=None,
                 override_icon_x : Optional[int]=None,
                ):
        bg_color = theme.bg_color
        super().__init__(master=master,
                         bg_color=bg_color,
                         width=width - resolve_padding(theme.padx, 0) - resolve_padding(theme.padx, 1),
                         height=theme.button_height)
        self.id = id
        self._selected = False
        self._hover = False
        self._theme = theme
        self._image_selected = None
        self._image = None
        self._click_commands = []
        self._select_commands = []
        self._deselect_commands = []
        self.has_submenu = has_submenu
        self.submenu_expanded = submenu_expanded

        if sys.platform == "darwin":
            self.configure(cursor="pointinghand")
        elif sys.platform.startswith("win"):
            self.configure(cursor="hand2")

        # Create canvas for custom drawing
        self._canvas = CTk.CTkCanvas(master=self,
                                 highlightthickness=0,
                                 width=self._apply_widget_scaling(self._desired_width),
                                 height=self._apply_widget_scaling(self._desired_height))
        self._draw_engine = CTk.DrawEngine(self._canvas)
        self._canvas.pack(fill="x", expand=True)

        # Label text
        self._text_label = CTk.CTkLabel(self, text=text, text_color=theme.text_color, bg_color="transparent")
        text_x = self._get_text_x(icon_size if icon else None) if override_text_x is None else override_text_x
        self._text_label.place(x=text_x, anchor="w", rely=0.5)

        # Hover handlers
        self._text_label.bind("<Enter>", self._on_enter)
        self._text_label.bind("<Leave>", self._on_leave)
        self._canvas.bind("<Enter>", self._on_enter)
        self._canvas.bind("<Leave>", self._on_leave)

        # Label icon
        self._label_image = None
        if icon:
            if isinstance(icon, Image.Image):
                self.image = CTk.CTkImage(light_image=colorize_image(self, icon, theme.text_color, 'light'),
                                        dark_image=colorize_image(self, icon, theme.text_color, 'dark'),
                                        size=icon_size)
                self.image_selected = CTk.CTkImage(light_image=colorize_image(self, icon, theme.text_color_selected, 'light'),
                                                dark_image=colorize_image(self, icon, theme.text_color_selected, 'dark'),
                                                size=icon_size)
            else:
                if not isinstance(icon, tuple) or len(icon) != 2 or not all(isinstance(i, CTk.CTkImage) for i in icon):
                    raise ValueError("Icon must be a PIL Image or a tuple of two CTkImage instances (light, dark).")
                self.image = icon[0]
                self.image_selected = icon[1]
            self._label_image = CTk.CTkLabel(self, text='', image=self.image, bg_color="transparent")
            icon_x = self._get_icon_x(icon_size if icon else None) if override_icon_x is None else override_icon_x
            self._label_image.place(x=icon_x, anchor="w", rely=0.5)
            self._label_image.bind("<Enter>", self._on_enter)
            self._label_image.bind("<Leave>", self._on_leave)
        self._draw() # Initial draw

    def bind_click(self, command):
        self._canvas.bind("<Button-1>", self._on_click)
        self._text_label.bind("<Button-1>", self._on_click)
        if self._label_image:
            self._label_image.bind("<Button-1>", self._on_click)
        self._click_commands.append(command)

    def bind_select(self, command):
        self._select_commands.append(command)
    
    def bind_deselect(self, command):
        self._deselect_commands.append(command)

    def select(self, call_bindings=True):
        if not self._selected:
            if len(self._select_commands) > 0 and call_bindings:
                for cmd in self._select_commands:
                    cmd()
            self._selected = True
            self._draw()

    def deselect(self, call_bindings=True):
        if self._selected:
            if len(self._deselect_commands) > 0 and call_bindings:
                for cmd in self._deselect_commands:
                    cmd()
            self._selected = False
            self._draw()

    def expand(self):
        if not self.submenu_expanded:
            self.submenu_expanded = True
            self._draw()
    
    def collapse(self):
        if self.submenu_expanded:
            self.submenu_expanded = False
            self._draw()

    def _set_scaling(self, *args, **kwargs):
        super()._set_scaling(*args, **kwargs)
        self._canvas.configure(width=self._apply_widget_scaling(self._desired_width),
                               height=self._apply_widget_scaling(self._desired_height))
        self._draw(no_color_updates=True)

    def _get_icon_x(self, icon_size: Optional[tuple[int, int]]) -> int:
        if self._theme.label_align_ref == "icon":
            # All buttons use the same icon x position, i.e. the indent of the theme
            return self._theme.label_indent
        else:
            # The reference is the left position of the text
            return self._theme.label_indent - icon_size[0] - self._theme.icon_text_margin
        
    def _get_text_x(self, icon_size: Optional[tuple[int, int]]) -> int:
        if self._theme.label_align_ref == "icon":
            # All buttons use the same icon x position, i.e. the indent of the theme
            if icon_size is None:
                return self._theme.label_indent
            else:
                return self._theme.label_indent + icon_size[0] + self._theme.icon_text_margin
        else:
            # The reference is the left position of the text
            return self._theme.label_indent

    def _draw(self, no_color_updates=False, update_idletasks=True):
        super()._draw(no_color_updates)
        requires_recoloring = False
        if self.has_submenu: # Draw dropdown arrow if a submenu exists
            requires_recoloring = self._draw_dropdown_arrow(self._apply_widget_scaling(self._current_width - (self._current_height / 2)),
                                                                      self._apply_widget_scaling(self._current_height / 2),
                                                                      self._apply_widget_scaling(8), self.submenu_expanded)
            requires_recoloring |= self._draw_selected_submenu_marker()
        requires_recoloring |= self._draw_engine.draw_rounded_rect_with_border(self._apply_widget_scaling(self._current_width), self._apply_widget_scaling(self._current_height), self._apply_widget_scaling(self._theme.button_corner_radius), 0)

        if no_color_updates is False or requires_recoloring:
            # Select the colors depending on the item's state
            bg_color = self._apply_appearance_mode(self._theme.bg_color)
            button_color = self._resolve_fg_color(self._theme.button_color, self._theme.bg_color)
            text_color = self._resolve_fg_color(self._theme.text_color, button_color)
            if self._selected and not self.has_submenu:
                button_color = self._resolve_fg_color(self._theme.button_color_selected, self._theme.bg_color)
                text_color = self._resolve_fg_color(self._theme.text_color_selected, button_color)
            elif self._hover:
                button_color = self._resolve_fg_color(self._theme.button_color_hover, bg_color)
                text_color = self._resolve_fg_color(self._theme.text_color_hover, button_color)
            self._text_label.configure(bg_color=button_color, text_color=text_color)

            # Update elements of an item with a submenu
            if self.has_submenu:
                marker_color = self._resolve_fg_color(self._theme.button_color_selected if self._selected else button_color, button_color)
                self._canvas.itemconfig("submenu_marker", fill=marker_color)
                if self._selected:
                    text_color = self._resolve_fg_color(self._theme.button_color_selected, button_color)
                self._canvas.itemconfig("dropdown_arrow", fill=text_color)

            # Update canvas items that create the rounded button effect
            self._canvas.itemconfig("border_parts",
                        outline=button_color,
                        fill=button_color)
            self._canvas.itemconfig("inner_parts",
                        outline=button_color,
                        fill=button_color)
            
            # Select which colored label image
            if self._label_image:
                self._label_image.configure(image=self.image_selected if self._selected and not self.has_submenu else self.image, bg_color=button_color)
            self._canvas.configure(bg=bg_color)
        if update_idletasks:
            self._canvas.update_idletasks()

    def _on_click(self, event):
        for cmd in self._click_commands:
            cmd()
        return "break"

    def _on_enter(self, event):
        self._hover = True
        self._draw(update_idletasks=False)
    
    def _on_leave(self, event):
        self._hover = False
        self._draw(update_idletasks=False)
    
    def cget(self, attribute_name: str) -> any:
        if attribute_name == "fg_color":
            return self._theme.button_color_selected if self._selected else self._theme.button_color
        elif attribute_name == "text_color":
            return self._theme.text_color_selected if self._selected else self._theme.text_color
        else:
            return super().cget(attribute_name)
        
    def _draw_dropdown_arrow(self, x_position: Union[int, float], y_position: Union[int, float], size: Union[int, float], point_down : bool = True) -> bool:
        x_position, y_position, size = round(x_position), round(y_position), size
        requires_recoloring = False

        if not self._canvas.find_withtag("dropdown_arrow"):
            self._canvas.create_line(0, 0, 0, 0, tags="dropdown_arrow", width=size / 3, joinstyle=tkinter.ROUND, capstyle=tkinter.ROUND)
            self._canvas.tag_raise("dropdown_arrow")
            requires_recoloring = True

        if point_down:
            self._canvas.coords("dropdown_arrow",
                            x_position - (size / 2),
                            y_position - (size / 5),
                            x_position,
                            y_position + (size / 5),
                            x_position + (size / 2),
                            y_position - (size / 5))
        else:
            self._canvas.coords("dropdown_arrow",
                            x_position - (size / 2),
                            y_position + (size / 5),
                            x_position,
                            y_position - (size / 5),
                            x_position + (size / 2),
                            y_position + (size / 5))

        return requires_recoloring
    
    def _draw_selected_submenu_marker(self) -> bool:
        requires_recoloring = False
        if not self._canvas.find_withtag("submenu_marker"):
            self._canvas.create_line(0, 0, 0, 0, tags="submenu_marker", width=self._apply_widget_scaling(self._theme.submenu_marker_thickness))
            self._canvas.tag_raise("submenu_marker")
            requires_recoloring = True
        self._canvas.coords("submenu_marker",
                                   self._apply_widget_scaling(self._theme.submenu_marker_padx),
                                   self._apply_widget_scaling(self._theme.submenu_marker_pady),
                                   self._apply_widget_scaling(self._theme.submenu_marker_padx),
                                   self._apply_widget_scaling(self._current_height-self._theme.submenu_marker_pady))
        return requires_recoloring

    def _resolve_fg_color(self, fg_color : Union[str, list], bg_color : Union[str, list]) -> str:
            resolved_fg = self._apply_appearance_mode(fg_color)
            resolved_bg = self._apply_appearance_mode(bg_color)
            if resolved_fg == "transparent":
                return resolved_bg
            return resolved_fg
    

class CTkSidebarSeparator(CTk.CTkBaseClass):
    def __init__(self,
                 master,
                 bg_color : Union[str, list],
                 width : int,
                 height : int,
                 line_length : int,
                 line_color : Union[str, list],
                 line_thickness : int,
                 rounded_line_end : bool = False
                ):
        super().__init__(master=master,
                         bg_color=bg_color,
                         width=width,
                         height=height)
        self._bg_color = bg_color
        self._line_color = line_color
        self._line_thickness = line_thickness
        self._line_length = line_length
        self._rounded_line_end = rounded_line_end
        self._canvas = None

        # Create canvas for custom drawing, if a line is specified
        self._canvas = CTk.CTkCanvas(master=self,
                                highlightthickness=0,
                                width=self._apply_widget_scaling(self._desired_width),
                                height=self._apply_widget_scaling(self._desired_height))
        self._canvas.pack(fill="x", expand=True)
        self._draw() # Initial draw

    def _draw(self, no_color_updates=False):
        super()._draw(no_color_updates)
        requires_recoloring = False
        if self._line_thickness > 0 and self._line_length > 0:
            self._draw_separator_line()
        if requires_recoloring or no_color_updates is False:
            line_color = self._apply_appearance_mode(self._line_color)
            self._canvas.itemconfig("separator",
                        fill=line_color)
            self._canvas.configure(bg=self._apply_appearance_mode(self._bg_color))
        self._canvas.update_idletasks()

    def _set_scaling(self, *args, **kwargs):
        super()._set_scaling(*args, **kwargs)
        self._canvas.configure(width=self._apply_widget_scaling(self._desired_width),
                               height=self._apply_widget_scaling(self._desired_height))
        self._draw(no_color_updates=True)

    def _draw_separator_line(self) -> bool:
        requires_recoloring = False
        if not self._canvas.find_withtag("separator"):
            self._canvas.create_line(0, 0, 0, 0, tags="separator", width=self._apply_widget_scaling(self._line_thickness), capstyle=tkinter.ROUND if self._rounded_line_end else tkinter.BUTT)
            self._canvas.tag_raise("separator")
            requires_recoloring = True
        self._canvas.coords("separator",
                                   self._apply_widget_scaling((self._current_width - self._line_length) / 2),
                                   self._apply_widget_scaling(self._current_height / 2),
                                   self._apply_widget_scaling((self._current_width + self._line_length) / 2),
                                   self._apply_widget_scaling(self._current_height / 2))
        return requires_recoloring


from typing import Literal, Optional, Union, Tuple, List
import customtkinter as CTk

class CTkSidebarTheme:
    def __init__(self,
                 load_default: Literal['primary', 'secondary']='primary',
                 bg_color: Optional[str]=None,
                 padx: Optional[int]=None,
                 pady: Optional[Union[int, Tuple[int, int]]]=None,
                 submenu_pady: Optional[Union[int, Tuple[int, int]]]=None,
                 button_color: Optional[Union[str, List[str]]]=None,
                 button_color_hover: Optional[Union[str, List[str]]]=None,
                 button_color_selected: Optional[Union[str, List[str]]]=None,
                 button_corner_radius: Optional[int]=None,
                 button_height: Optional[int]=None,
                 text_color: Optional[Union[str, List[str]]]=None,
                 text_color_hover: Optional[Union[str, List[str]]]=None,
                 text_color_selected: Optional[Union[str, List[str]]]=None,
                 label_indent: Optional[int]=None,
                 label_indent_increment: Optional[int]=None,
                 label_align_ref: Optional[Literal['text', 'icon']]=None,
                 icon_text_margin: Optional[int]=None,
                 separator_line_color: Optional[Union[str, List[str]]]=None,
                 separator_line_thickness: Optional[int]=None,
                 separator_height: Optional[int]=None,
                 separator_width: Optional[int]=None,
                 separator_rounded_line_end: Optional[bool]=None,
                 submenu_marker_thickness: Optional[Union[str, List[str]]]=None,
                 submenu_marker_padx: Optional[int]=None,
                 submenu_marker_pady: Optional[int]=None,
                ):
        """
        Theme configuration for CTkSidebar and CTkSidebarItem.
        Parameters:
            load_default: 'primary' for main sidebar items, 'secondary' for submenu items.
            bg_color: Background color of the sidebar.
            padx: Horizontal padding inside the sidebar frame.
            pady: Vertical padding inside the sidebar frame.
            submenu_pady: Vertical padding before and after submenu sections. If desired, you can specify a tuple for (top, bottom) padding.
            button_color: Background color of unselected sidebar buttons.
            button_color_hover: Background color when hovering over buttons.
            button_color_selected: Background color of the selected button.
            button_corner_radius: Corner radius of sidebar buttons.
            button_height: Height of sidebar buttons.
            text_color: Text color of unselected sidebar buttons.
            text_color_hover: Text color when hovering over buttons.
            text_color_selected: Text color of the selected button.
            label_indent: Base indentation for sidebar item labels.
            label_indent_increment: Additional indentation per submenu level.
            label_align_ref: Horizontally align labels by their text or icon.
            icon_text_margin: Margin between icon and text in sidebar items.
            separator_line_color: Color of separator lines.
            separator_line_thickness: Thickness of separator lines.
            separator_height: Total height of a separator item.
            separator_width: Width of the (centered) separator line.
            separator_rounded_line_end: Whether separator lines have rounded or 'butt' ends.
            submenu_marker_thickness: Thickness of the submenu expansion marker.
            submenu_marker_padx: Horizontal offset of the submenu marker with respect to the button's left edge.
            submenu_marker_pady: Vertical padding of the submenu marker with respect to the button's top and bottom edges.
        """
        # Load defaults
        default = CTk.ThemeManager.theme
        if load_default == 'primary':
            # Default theme for the 'primary' style (non-submenu items)
            self.bg_color = default["CTkFrame"]["fg_color"]
            self.padx = 10
            self.pady = (10,10)
            self.submenu_pady = (10,10)
            self.button_color = "transparent"
            self.button_color_hover = default["CTkFrame"]["top_fg_color"]
            self.button_color_selected = default["CTkButton"]["fg_color"]
            self.button_corner_radius = 5
            self.button_height = 36
            self.text_color = default["CTkLabel"]["text_color"]
            self.text_color_hover = default["CTkLabel"]["text_color"]
            self.text_color_selected = default["CTkButton"]["text_color"]
            self.label_indent = 50
            self.label_indent_increment = 0
            self.label_align_ref = 'text'
            self.icon_text_margin = 10
            self.separator_line_color = default["CTkEntry"]["border_color"]
            self.separator_line_thickness = 3
            self.separator_height = 15
            self.separator_width = 20
            self.separator_rounded_line_end = True
            self.submenu_marker_thickness = 3
            self.submenu_marker_padx = 5
            self.submenu_marker_pady = 5
        else:
            # Default theme for the 'secondary' style (submenu items)
            self.bg_color = default["CTkFrame"]["top_fg_color"]
            self.padx = 10
            self.pady = (8,8)
            self.submenu_pady = (2,2)
            self.button_color = "transparent"
            self.button_color_hover = ["gray84", "gray24"]
            self.button_color_selected = default["CTkButton"]["fg_color"]
            self.button_corner_radius = 5
            self.button_height = 32
            self.text_color = ["gray30", "gray70"]
            self.text_color_hover = ["gray30", "gray70"]
            self.text_color_selected = default["CTkButton"]["text_color"]
            self.label_indent = 30
            self.label_indent_increment = 20
            self.label_align_ref = 'text'
            self.icon_text_margin = 10
            self.separator_line_color = ["gray84", "gray24"]
            self.separator_line_thickness = 2
            self.separator_height = 10
            self.separator_width = 50
            self.separator_rounded_line_end = False
            self.submenu_marker_thickness = 2
            self.submenu_marker_padx = 8
            self.submenu_marker_pady = 6

        # Then override defaults with provided values
        if bg_color is not None:
            self.bg_color = bg_color
        if padx is not None:
            self.padx = padx
        if pady is not None:
            self.pady = pady
        if submenu_pady is not None:
            self.submenu_pady = submenu_pady
        if button_color is not None:
            self.button_color = button_color
        if button_color_hover is not None:
            self.button_color_hover = button_color_hover
        if button_color_selected is not None:
            self.button_color_selected = button_color_selected
        if button_corner_radius is not None:
            self.button_corner_radius = button_corner_radius
        if button_height is not None:
            self.button_height = button_height
        if text_color is not None:
            self.text_color = text_color
        if text_color_hover is not None:
            self.text_color_hover = text_color_hover
        if text_color_selected is not None:
            self.text_color_selected = text_color_selected
        if label_indent is not None:
            self.label_indent = label_indent
        if icon_text_margin is not None:
            self.icon_text_margin = icon_text_margin
        if label_indent_increment is not None:
            self.label_indent_increment = label_indent_increment
        if separator_line_color is not None:
            self.separator_line_color = separator_line_color
        if separator_line_thickness is not None:
            self.separator_line_thickness = separator_line_thickness
        if separator_height is not None:
            self.separator_height = separator_height
        if separator_width is not None:
            self.separator_width = separator_width
        if separator_rounded_line_end is not None:
            self.separator_rounded_line_end = separator_rounded_line_end
        if label_align_ref is not None:
            self.label_align_ref = label_align_ref
        if submenu_marker_thickness is not None:
            self.submenu_marker_thickness = submenu_marker_thickness
        if submenu_marker_padx is not None:
            self.submenu_marker_padx = submenu_marker_padx
        if submenu_marker_pady is not None:
            self.submenu_marker_pady = submenu_marker_pady


from typing import Callable, Optional, Union
import customtkinter as CTk
import copy
from .util import resolve_padding
from .theme import CTkSidebarTheme
from .item import CTkSidebarItem, CTkSidebarSeparator
from PIL import Image

class CTkSidebar(CTk.CTkFrame):
    def __init__(self, master=None, width: int=220, theme: Optional[CTkSidebarTheme]=None, indent_level: int=0, single_expanded_submenu: bool=False):
        self._change_commands : list[Callable[[Optional[Union[str, int]]], None]] = []
        self._add_item_commands : list[Callable[[Optional[Union[str, int]]], None]] = []
        self._visible = True
        self._is_submenu = False
        self._parent_sidebar_item = None
        self._parent_menu = None
        self._children : list[CTk.CTkBaseClass] = []
        self._indent_level = indent_level
        self._width : Optional[int] = width
        self._single_expanded_submenu = single_expanded_submenu
        self.selected_item : Optional[CTkSidebarItem] = None
        self._content = self
        if theme:
            self._theme = copy.copy(theme)
        else: # Use default theme if none is provided
            self._theme = CTkSidebarTheme(load_default='primary')
        self._theme.label_indent += self._indent_level * self._theme.label_indent_increment
        super().__init__(master, width=width, corner_radius=0, fg_color=self._theme.bg_color)

    def _set_parent(self, parent_menu: "CTkSidebar", parent_item: CTkSidebarItem):
        self._is_submenu = True
        self._parent_menu = parent_menu
        self._parent_sidebar_item = parent_item
    
    def add_frame(self, frame: CTk.CTkBaseClass, pady: Optional[Union[int, tuple[int,int]]]=None) -> None:
        self._children.append(frame)
        self._children[-1].grid(row=len(self._children)-1, column=0, sticky="ew", padx=self._theme.padx, pady=pady if pady is not None else 0)

    def add_item(self,
                 id : Optional[Union[int, str]]=None,
                 text: str="",
                 command: Optional[Callable[[Union[int, str]], None]]=None,
                 icon: Optional[Union[Image.Image, tuple[CTk.CTkImage, CTk.CTkImage]]]=None,
                 icon_size : tuple[int, int]=(20,20),
                 override_text_x: Optional[int]=None,
                 override_icon_x: Optional[int]=None
                ) -> None:
        if len(self._children) == 0 and not self._is_submenu and (pady := resolve_padding(self._theme.pady, 0)) > 0:
            self.add_spacing(pady) # Add initial top padding

        ctk_item = CTkSidebarItem(self,
                                  theme=self._theme,
                                  id=id,
                                  text=text,
                                  width=self._width,
                                  icon=icon,
                                  icon_size=icon_size,
                                  has_submenu=False,
                                  submenu_expanded=False,
                                  override_text_x=override_text_x,
                                  override_icon_x=override_icon_x
                                 )
        self._children.append(ctk_item)
        self._children[-1].grid(row=len(self._children)-1, column=0, sticky="ew", padx=self._theme.padx, pady=0)

        ctk_item.bind_click(lambda _=None, id=id: self._on_click(id))
        ctk_item.bind_click(lambda _=None, item=ctk_item: self._select_item(item))
        if command:
            ctk_item.bind_click(lambda _=None, cmd=command, id=id: cmd(id))
        self._on_add_item(id)

    def add_submenu(self, 
                    id : Optional[Union[int, str]]=None,
                    text: str="",
                    command: Optional[Callable[[Union[int, str]], None]]=None,
                    icon: Optional[Union[Image.Image, tuple[CTk.CTkImage, CTk.CTkImage]]]=None,
                    icon_size : tuple[int, int]=(20,20),
                    override_text_x: Optional[int]=None,
                    override_icon_x: Optional[int]=None,
                    indent_level: Optional[int]=None,
                    theme: Optional[CTkSidebarTheme]=None,
                    expanded: bool=True,
                   ) -> "CTkSidebar":
        if len(self._children) == 0 and not self._is_submenu and (pady := resolve_padding(self._theme.pady, 0)) > 0:
            self.add_spacing(pady)

        # Create an item on the current indentation level, with a dropdown
        ctk_item = CTkSidebarItem(self,
                                  theme=self._theme,
                                  id=id,
                                  text=text,
                                  width=self._width,
                                  icon=icon,
                                  icon_size=icon_size,
                                  has_submenu=True,
                                  submenu_expanded=True,
                                  override_text_x=override_text_x,
                                  override_icon_x=override_icon_x
                                 )
        self._children.append(ctk_item)
        self._children[-1].grid(row=len(self._children)-1, column=0, sticky="ew", padx=self._theme.padx, pady=0)

        # Create a new submenu with increased indentation level
        submenu_theme = theme if theme else CTkSidebarTheme(load_default='secondary')
        wrapper = CTkSidebarSubmenu(self, width=self._width, theme=submenu_theme, indent_level=(self._indent_level + 1 if indent_level is None else indent_level))
        submenu = wrapper.sidebar
        submenu._set_parent(self, ctk_item)
        self._children.append(wrapper)
        self._children[-1].grid(row=len(self._children)-1, column=0, sticky="ew", padx=0, pady=self._theme.submenu_pady)
        ctk_item.bind_click(lambda _=None, item=ctk_item, wrapper=wrapper: self._toggle_submenu(item, wrapper))
        ctk_item.bind_select(lambda _=None, item=ctk_item: self._select_item(item))
        ctk_item.bind_deselect(lambda _=None, submenu=submenu: submenu._deselect())
        if command:
            ctk_item.bind_click(lambda _=None, cmd=command, id=id: cmd(id))
        if not expanded:
            wrapper.hide()
            ctk_item.collapse()
        self._on_add_item(id)
        return submenu
    
    def add_separator(self,
                      width: Optional[int]=None,
                      height: Optional[int]=None,
                      line_color: Optional[Union[str, list[str]]]=None,
                      line_thickness: Optional[int]=None,
                      rounded_line_end: Optional[bool]=None,
                     ) -> None:
        separator = CTkSidebarSeparator(master=self, width=self._width,
                                        bg_color=self._theme.bg_color,
                                        height=height if height is not None else self._theme.separator_height,
                                        line_length=width if width is not None else self._theme.separator_width,
                                        line_color=line_color if line_color is not None else self._theme.separator_line_color,
                                        line_thickness=line_thickness if line_thickness is not None else self._theme.separator_line_thickness,
                                        rounded_line_end=rounded_line_end if rounded_line_end is not None else self._theme.separator_rounded_line_end
                                       )
        self._children.append(separator)
        self._children[-1].grid(row=len(self._children)-1, column=0, sticky="ew", padx=0, pady=0)

    def add_spacing(self, height: int) -> None:
        self.add_separator(height=height, line_thickness=-1)

    def _select_item(self, item : CTkSidebarItem):
        if self.selected_item and self.selected_item != item:
            self.selected_item.deselect()
        item.select(False)
        self.selected_item = item
        if self._parent_sidebar_item:
            self._parent_sidebar_item.select()

    def _deselect(self):
        if self.selected_item:
            self.selected_item.deselect()
        self.selected_item = None

    def get_item(self, id: Union[int, str]) -> Optional[CTkSidebarItem]:
        for child in self._children:
            if isinstance(child, CTkSidebarItem):
                if child.id == id:
                    return child
            elif isinstance(child, CTkSidebar):
                if sub_item := child.get_item(id):
                    return sub_item
        return None

    def bind_change(self, command: Callable[[Optional[Union[str, int]]], None], overwrite: bool=False):
        if overwrite:
            self._change_commands = [command]
        else:
            self._change_commands.append(command)

    def bind_add_item(self, command: Callable[[Optional[Union[str, int]]], None], overwrite: bool=False):
        if overwrite:
            self._add_item_commands = [command]
        else:
            self._add_item_commands.append(command)

    def _toggle_submenu(self, item : CTkSidebarItem, submenu : "CTkSidebarSubmenu"):
        if item.submenu_expanded:
            item.collapse()
            submenu.hide()
        else:
            item.expand()
            submenu.show()
            if self._single_expanded_submenu:
                # Collapse any other expanded submenu
                for child in self._children:
                    if isinstance(child, CTkSidebarSubmenu) and child != submenu and child._visible:
                        child.hide()
                        child.sidebar._parent_sidebar_item.collapse()
        self._draw()

    def _on_add_item(self, id: Optional[Union[str, int]]=None):
        if self._parent_menu:
            self._parent_menu._on_add_item(id)
        for callback in self._add_item_commands:
            callback(id)

    def _draw(self, no_color_updates=False):
        super()._draw(no_color_updates)

    def _on_click(self, id: Optional[Union[str, int]]=None):
        if self._parent_menu:
            self._parent_menu._on_click(id)
        else:
            for cmd in self._change_commands:
                cmd(id)

class CTkSidebarSubmenu(CTk.CTkFrame):
    def __init__(self, master=None, width: int=220, theme: CTkSidebarTheme=None, indent_level: int=0, single_expanded_submenu: bool=False):
        super().__init__(master, width=width, corner_radius=0, fg_color=theme.bg_color)
        self.sidebar = CTkSidebar(master=self, width=width, theme=theme, indent_level=indent_level, single_expanded_submenu=single_expanded_submenu)
        self.sidebar.grid(row=0, column=0, sticky="nsew", padx=0, pady=theme.pady)
        self._visible = True
    
    def _set_scaling(self, *args, **kwargs):
        super()._set_scaling(*args, **kwargs)
        if not self._visible:
            self.grid_remove()

    def hide(self):
        if self._visible:
            self.grid_remove()
            self._visible = False

    def show(self):
        if not self._visible:
            self.grid()
            self._visible = True


import customtkinter as CTk
from typing import Any, Optional, Union
from .menu import CTkSidebar
from .theme import CTkSidebarTheme

class CTkSidebarNavigation(CTk.CTkFrame):
    def __init__(self, master: Any, width: int=220, theme: Optional[CTkSidebarTheme]=None, indent_level: int=0, single_expanded_submenu: bool=False):
        super().__init__(master, fg_color="transparent")
        self._tabs = {}
        self._current_id = None
        self.sidebar : Optional[CTkSidebar] = None
        self.set_sidebar(CTkSidebar(master=self, width=width, theme=theme, indent_level=indent_level, single_expanded_submenu=single_expanded_submenu))

    def set_sidebar(self, sidebar: CTkSidebar):
        if self.sidebar:
            self.sidebar.pack_forget()
        self.sidebar = sidebar
        self.sidebar.pack(side="left", fill="y")
        self.sidebar.bind_change(self._on_change, overwrite=True)
        self.sidebar.bind_add_item(self._on_add_item, overwrite=True)

    def view(self, id: Union[int, str]) -> CTk.CTkFrame:
        """Returns the frame associated with the given ID."""
        item = self._tabs.get(id)
        if item:
            return item
        else:
            raise ValueError(f"Sidebar has no item ID '{id}'")
        
    def set(self, id: Union[int, str]):
        """Sets the current view to the frame associated with the given ID."""
        item = self.sidebar.get_item(id)
        if not item:
            raise ValueError(f"Sidebar has no item ID '{id}'")
        item._on_click(None)

    def _on_add_item(self, id: Union[int, str]) -> None:
        if id is None:
            raise ValueError(f"When using CTkSidebarNavigation, all items must have a non-empty ID. Item: '{item.text}'")
        if self._tabs.get(id):
            raise ValueError(f"Duplicate sidebar item ID: {id}. All item IDs within a CTkSidebarNavigation must be unique.")
        self._tabs[id] = CTk.CTkFrame(master=self, fg_color="transparent", corner_radius=0)
    
    def _on_change(self, id: Optional[Union[int, str]]):
        if id in self._tabs:
            # Remove previous tab if it exists
            if self._current_id and self._current_id in self._tabs:
                self._tabs[self._current_id].pack_forget()
            # Show new tab
            tab = self._tabs[id]
            tab.pack(side="left", fill="both", expand=True)
            tab.pack_propagate(False)
            self._current_id = id
